@@ -22,14 +22,16 @@ has_postgres_client() {
     which psql > /dev/null 2>&1
 }
 
-authenumerate_as_courier() {
-    su -c "/usr/sbin/authenumerate" -s /bin/sh courier
-}
-
-authtest_as_courier() {
+test_authentication() {
     user=$1
     password=$2
-    su -c "/usr/sbin/authtest $user $password" -s /bin/sh courier
+    TEST_OUTPUT="$AUTOPKGTEST_ARTIFACTS/testauth-$1.out"
+    echo "testing: '$user' with password '$password'"
+    /usr/sbin/authtest $user $password > $TEST_OUTPUT
+}
+
+authenumerate_as_courier() {
+    su -c "/usr/sbin/authenumerate" -s /bin/sh courier
 }
 
 # emits a random (512bit, hex encoded) password on stdout
@@ -43,97 +45,102 @@ postgres_superuser_exec() {
 }
 
 create_test_users() {
-    echo "create test users..."
-    PASSWORD_ALICE=$(gen_random_password)
-    PASSWORD_BOB=$(gen_random_password)
-    PASSWORD_CAROL=$(gen_random_password)
-
-    useradd --shell /bin/false --password $PASSWORD_ALICE alice
-    useradd --shell /bin/false --password $PASSWORD_BOB   bob
-    useradd --shell /bin/false --password $PASSWORD_CAROL carol
-
-    ALICE_UID=$(id -u alice)
-    ALICE_GID=$(id -g alice)
-    BOB_UID=$(id -u bob)
-    BOB_GID=$(id -g bob)
-    CAROL_UID=$(id -u carol)
-    CAROL_GID=$(id -g carol)
+    echo "== creating test users..."
+    for USER in $TEST_USERS; do
+        gen_random_password > $USER.password
+        useradd --shell /bin/false --password $(cat $USER.password) $USER
+    done
 }
 
 backup_config_files() {
-    echo "backup config files..."
+    echo "== backup config files..."
     for f in $CONFIG_FILES; do
-	if [ -f $f ]; then
-	    cp ${f} ${f}.autopkgtest.bak
-	fi
+        if [ -f $f ]; then
+            cp ${f} ${f}.autopkgtest.bak
+        fi
     done
 }
 
 restore_config_files() {
-    echo "restore config files..."
+    echo "== restore config files..."
     for f in $CONFIG_FILES; do
-	if [ -f ${f}.autopkgtest.bak ]; then
-	    mv ${f}.autopkgtest.bak ${f}
-	fi
+        if [ -f ${f}.autopkgtest.bak ]; then
+            mv ${f}.autopkgtest.bak ${f}
+        fi
     done
 }
 
-# cleanup after running tests
-finish() {
-    echo "finish..."
-    # restore config files, then restart the authdaemon, so it
-    # disconnects from the database. Otherwise authdaemon blocks the
-    # database deletion.
-    restore_config_files
-    service courier-authdaemon restart
-
-    # cleanup Postgres databases
-    if has_postgres_client; then
-        postgres_superuser_exec <<EOSQL
-DROP DATABASE IF EXISTS test;
-DROP ROLE IF EXISTS courier;
-EOSQL
-    fi
-
-    # drop test users
-    if user_exists alice; then
-	echo "dropping user alice"
-        userdel alice
-    fi
-    if user_exists bob; then
-	echo "dropping user bob"
-        userdel bob
-    fi
-    if user_exists carol; then
-	echo "dropping user carol"
-	userdel carol
-    fi
+start_authdaemon() {
+    echo "== starting authdameon..."
+    service courier-authdaemon start
 }
-trap finish EXIT
 
+start_postgresql() {
+    echo "== starting postgresql..."
+    service postgresql start
+}
+
+# helper methods for dumping test status
 dump_file_if_exists() {
     if [ -f $1 ]; then
-	echo "===== BEGIN $1 ====="
-	cat $1
-	echo "===== END $1 ====="
+        echo "===== BEGIN $1 ====="
+        cat $1
+        echo "===== END $1 ====="
     fi
 }
 
 dump_config_files() {
     for f in $CONFIG_FILES; do
-	if [ -f ${f}.autopkgtest.bak ]; then
-	    dump_file_if_exists $f
-	fi
+        if [ -f ${f}.autopkgtest.bak ]; then
+            dump_file_if_exists $f
+        fi
+    done
+
+    for f in `ls $AUTOPKGTEST_ARTIFACTS/`; do
+        dump_file_if_exists $AUTOPKGTEST_ARTIFACTS/$f
     done
 }
 
-
-dump_environment() {
-    echo "dump..."
+# cleanup after running tests
+finish() {
+    echo "== dump..."
     # dump and then restore the config files
     dump_config_files
 
-    # cleanup as well, especially after an error
-    finish
+    echo "== finish..."
+
+    # drop test users
+    if user_exists alice; then
+        echo "== dropping user alice"
+        userdel alice
+    fi
+    if user_exists bob; then
+        echo "== dropping user bob"
+        userdel bob
+    fi
+    if user_exists carol; then
+        echo "== dropping user carol"
+        userdel carol
+    fi
+
+    # restore config files, then restart the authdaemon, so it
+    # disconnects from the database. Otherwise authdaemon blocks the
+    # database deletion.
+    restore_config_files
+
+    # cleanup Postgres databases
+    if has_postgres_client; then
+        postgres_superuser_exec <<EOSQL
+DROP DATABASE IF EXISTS courier_authdaemon_test;
+DROP ROLE IF EXISTS courier;
+EOSQL
+    fi
+
+    for NAME in courier-authdaemon postgresql; do
+        if [ -x /etc/init.d/$NAME ]; then
+            echo "== stopping service $NAME..."
+            service $NAME stop || /bin/true
+        fi
+    done
 }
-trap dump_environment INT QUIT ABRT PIPE TERM
+trap finish EXIT INT QUIT ABRT PIPE TERM
